#include "precompile.h"

#include <google/protobuf/message.h>
#include <a8/websocketsession.h>
#include <a8/tcplistener.h>

#include "framework/cpp/netmsghandler.h"

#include "app.h"
#include "GSListener.h"
#include "jsondatamgr.h"
#include "ss_proto.pb.h"
#include "handlermgr.h"

class GSClientSession: public a8::WebSocketSession
{
public:

    virtual void DecodeUserPacket(char* buf, int& offset, unsigned int buflen) override
    {
        //packagelen + msgid + magiccode + msgbody
        //2 + 2 + 4+ xx + \0 + xx
        bool warning = false;
        while (buflen - offset >= sizeof(PackHead)) {
            PackHead* p = (PackHead*)&buf[offset];
            if (p->magic_code == MAGIC_CODE) {
                if (buflen - offset < sizeof(PackHead) + p->packlen) {
                    break;
                }
                App::Instance()->AddSocketMsg(SF_Client,
                                              socket_handle,
                                              saddr,
                                              p->msgid,
                                              p->seqid,
                                              &buf[offset + sizeof(PackHead)],
                                              p->packlen);
                offset += sizeof(PackHead) + p->packlen;
            } else {
                warning = true;
                offset++;
                continue;
            }
        }

        if (warning) {
            a8::UdpLog::Instance()->Warning("收到client非法数据包", {});
        }
    }

    virtual void OnRawHttpGet(const std::string& url, const std::string& querystr,
                              std::string& response) override
    {
        App::Instance()->AddIMMsg(IM_ExecGM,
                                  a8::XParams()
                                  .SetSender(socket_handle)
                                  .SetParam1(url)
                                  .SetParam2(querystr));
    }

    virtual void OnDisConnect() override
    {
        App::Instance()->AddIMMsg(IM_ClientSocketDisconnect,
                                  a8::XParams()
                                  .SetSender(socket_handle)
                                  .SetParam1(1));
    }

};

static void CreateGameClientSocket(a8::TcpSession **p)
{
    *p = new GSClientSession();
}

static void GSListeneron_error(a8::TcpListener*, int type, int errorid)
{
    a8::UdpLog::Instance()->Debug("GSListeneron_error %d %d", {type, errorid});
}

void GSListener::Init()
{
    tcp_listener_ = new a8::TcpListener();
    tcp_listener_->on_create_client_socket = CreateGameClientSocket;
    tcp_listener_->on_error = GSListeneron_error;

    tcp_listener_->bind_address = "0.0.0.0";
    tcp_listener_->bind_port = JsonDataMgr::Instance()->GetConf()->At("listen_port")->AsXValue();
    tcp_listener_->Open();
}

void GSListener::UnInit()
{
    delete tcp_listener_;
    tcp_listener_ = nullptr;
}

void GSListener::ForwardTargetConnMsg(MsgHdr& hdr)
{
    char* buff = (char*)malloc(sizeof(PackHead) + hdr.buflen);
    PackHead* head = (PackHead*)buff;
    head->packlen = hdr.buflen;
    head->msgid = hdr.msgid;
    head->seqid = hdr.seqid;
    head->magic_code = MAGIC_CODE;
    head->rpc_error_code = 0;
    if (hdr.buflen > 0) {
        memmove(buff + sizeof(PackHead), hdr.buf, hdr.buflen);
    }

    tcp_listener_->SendClientMsg(hdr.socket_handle, buff, sizeof(PackHead) + head->packlen);
    free(buff);
}

void GSListener::SendText(unsigned short sockhandle, const std::string& text)
{
    tcp_listener_->SendClientMsg(sockhandle, text.data(), text.size());
}

void GSListener::ForceCloseClient(unsigned short sockhandle)
{
    tcp_listener_->ForceCloseClient(sockhandle);
}

void GSListener::MarkClient(unsigned short sockhandle, bool is_active)
{
    tcp_listener_->MarkClient(sockhandle, is_active);
}
